package memory

import (
	"context"
	"errors"
	"slices"

	"go.opentelemetry.io/otel/attribute"
	"go.opentelemetry.io/otel/codes"

	pb "gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/api/v1alpha1/weather"
	"gitea.libretechconsulting.com/rmcguire/ambient-local-exporter/pkg/weather"
)

func (r *MemoryRecorder) Get(ctx context.Context, req *pb.GetWeatherRequest) (
	[]*weather.WeatherUpdate, error,
) {
	ctx, span := r.tracer.Start(ctx, "memoryRecorder.Get")
	defer span.End()

	r.RLock()
	defer r.RUnlock()

	span.AddEvent("acquired lock on recorder cache")

	updates := r.updates

	if r.count() == 0 {
		err := errors.New("no recorded updates to get")
		span.RecordError(err)
		return nil, err
	} else if r.count() <= int(*req.Limit) {
		span.RecordError(errors.New("requested more updates than recorded"))
	}

	// Filter by Station Name if requested
	if req.Opts.StationName != nil && *req.Opts.StationName != "" {
		updates = slices.DeleteFunc(updates, func(u *weather.WeatherUpdate) bool {
			return u.StationConfig == nil || u.StationConfig.Name != *req.Opts.StationName
		})
	}

	// Filter by Station Type if requested
	if req.Opts.StationType != nil && *req.Opts.StationType != "" {
		updates = slices.DeleteFunc(updates, func(u *weather.WeatherUpdate) bool {
			return u.StationType == nil || *u.StationType != *req.Opts.StationType
		})
	}

	// Limit results
	if len(updates) > int(*req.Limit) {
		updates = updates[len(updates)-int(*req.Limit):]
	}

	span.SetAttributes(attribute.Int("retrieved", len(updates)))
	span.SetStatus(codes.Ok, "")

	return updates, nil
}

func (r *MemoryRecorder) Count(ctx context.Context) int {
	_, span := r.tracer.Start(ctx, "countWeatherRecorder")
	defer span.End()

	count := r.count()

	span.SetAttributes(attribute.Int("count", count))
	span.SetStatus(codes.Ok, "")

	return count
}

func (r *MemoryRecorder) count() int {
	return len(r.updates)
}
